@@ -35,10 +35,9 @@
 #include "tab.h"
 #include "preferences.h"
 #include "unixsocket.h"
-#include "encoding.h"
 
 /* Linux color for palette */
-const GdkColor linux_color[16] =
+static const GdkColor linux_color[16] =
 {
   { 0, 0x0000, 0x0000, 0x0000 },
   { 0, 0xaaaa, 0x0000, 0x0000 },
@@ -58,8 +57,10 @@ const GdkColor linux_color[16] =
   { 0, 0xffff, 0xffff, 0xffff }
 };
 
-LXTerminal *lxterminal_init(LXTermWindow *lxtermwin, gint argc, gchar **argv, Setting *setting);
-void terminal_childexit(VteTerminal *vte, Term *term);
+static void terminal_childexit(VteTerminal *vte, Term *term);
+
+static Term *terminal_new(LXTerminal *terminal, const gchar *label, const gchar *pwd, gchar **env, const gchar *exec);
+static void terminal_free(Term* term);
 
 /* menu accel saved when the user disables it */
 static char *saved_menu_accel = NULL;
@@ -72,10 +73,9 @@ static gchar helpmsg[] = {
 	"  -t, -T, --title=STRING               Set the terminal's title\n"
 	"  --working-directory=DIRECTORY    Set the terminal's working directory\n"
 	"  --geometry=GEOMETRY              X geometry specification (see \"X\" man page), can be specified once per window to be opened.\n"
+	"  -l, --loginshell                 Executes login shell.\n"
 };
 
-#define MENUBAR_MENU_COUNT 4
-//#define MENUBAR_MENU_COUNT 5
 static GtkActionEntry menus[] =
 {
 	{ "File", NULL, N_("_File") },
@@ -84,46 +84,7 @@ static GtkActionEntry menus[] =
 	{ "Tabs", NULL, N_("_Tabs") },
 	{ "Help", NULL, N_("_Help") }
 };
-
-#define MENUBAR_MENUITEM_COUNT 14
-//#define MENUBAR_MENUITEM_COUNT 15
-static GtkActionEntry menu_items[] =
-{
-	{ "File_NewWindow", GTK_STOCK_ADD, N_("New _Window"), NEW_WINDOW_ACCEL, "New Window", G_CALLBACK(terminal_newwindow)},
-	{ "File_NewTab", GTK_STOCK_ADD, N_("New _Tab"), NEW_TAB_ACCEL, "New Tab", G_CALLBACK(terminal_newtab)},
-	{ "File_Sep1", NULL, "Sep" },
-	{ "File_CloseTab", GTK_STOCK_CLOSE, N_("_Close Tab"), CLOSE_TAB_ACCEL, "Close Tab", G_CALLBACK(terminal_closetab)},
-	{ "File_Quit", GTK_STOCK_QUIT, N_("_Quit"), QUIT_ACCEL, "Quit", G_CALLBACK(gtk_main_quit)},
-	{ "Edit_Copy", GTK_STOCK_COPY, N_("Cop_y"), COPY_ACCEL, "Copy", G_CALLBACK(terminal_copy)},
-	{ "Edit_Paste", GTK_STOCK_PASTE, N_("_Paste"), PASTE_ACCEL, "Paste", G_CALLBACK(terminal_paste)},
-	{ "Edit_Sep1", NULL, "Sep" },
-	{ "Edit_Preferences", GTK_STOCK_EXECUTE, N_("Preference_s"), NULL, "Preferences", G_CALLBACK(lxterminal_preferences_dialog)},
-//	{ "View_CharacterEncoding", NULL, N_("_Character Encoding"), NULL, "Character Encoding", NULL},
-	{ "Tabs_PreviousTab", GTK_STOCK_GO_BACK, N_("Pre_vious Tab"), PREVIOUS_TAB_ACCEL, "Previous Tab", G_CALLBACK(terminal_prevtab)},
-	{ "Tabs_NextTab", GTK_STOCK_GO_FORWARD, N_("Ne_xt Tab"), NEXT_TAB_ACCEL, "Next Tab", G_CALLBACK(terminal_nexttab)},
-	{ "Tabs_MoveTabLeft", NULL, N_("Move Tab _Left"), MOVE_TAB_LEFT_ACCEL, "Move Tab Left", G_CALLBACK(terminal_movetableft)},
-	{ "Tabs_MoveTabRight", NULL, N_("Move Tab _Right"), MOVE_TAB_RIGHT_ACCEL, "Move Tab Right", G_CALLBACK(terminal_movetabright)},
-	{ "Help_About", GTK_STOCK_ABOUT, N_("_About"), NULL, "About", G_CALLBACK(terminal_about)}
-};
-
-#define VTE_MENUITEM_COUNT 14
-static GtkActionEntry vte_menu_items[] =
-{
-	{ "VTEMenu", NULL, "VTEMenu" },
-	{ "NewWindow", GTK_STOCK_ADD, N_("New _Window"), NULL, "New Window", G_CALLBACK(terminal_newwindow)},
-	{ "NewTab", GTK_STOCK_ADD, N_("New _Tab"), NULL, "New Tab", G_CALLBACK(terminal_newtab)},
-	{ "Sep1", NULL, "Sep" },
-	{ "Copy", GTK_STOCK_COPY, N_("Cop_y"), NULL, "Copy", G_CALLBACK(terminal_copy)},
-	{ "Paste", GTK_STOCK_PASTE, N_("_Paste"), NULL, "Paste", G_CALLBACK(terminal_paste)},
-	{ "Sep2", NULL, "Sep" },
-	{ "Preferences", GTK_STOCK_EXECUTE, N_("Preference_s"), NULL, "Preferences", G_CALLBACK(lxterminal_preferences_dialog)},
-	{ "Sep3", NULL, "Sep" },
-	{ "PreviousTab", GTK_STOCK_GO_BACK, N_("Pre_vious Tab"), NULL, "Previous Tab", G_CALLBACK(terminal_prevtab)},
-	{ "NextTab", GTK_STOCK_GO_FORWARD, N_("Ne_xt Tab"), NULL, "Next Tab", G_CALLBACK(terminal_nexttab)},
-	{ "Tabs_MoveTabLeft", NULL, N_("Move Tab _Left"), NULL, "Move Tab Left", G_CALLBACK(terminal_prevtab)},
-	{ "Tabs_MoveTabRight", NULL, N_("Move Tab _Right"), NULL, "Move Tab Right", G_CALLBACK(terminal_nexttab)},
-	{ "CloseTab", GTK_STOCK_CLOSE, N_("_Close Tab"), NULL, "Close Tab", G_CALLBACK(terminal_closetab)}
-};
+#define MENUBAR_MENU_COUNT G_N_ELEMENTS(menus)
 
 static void
 gdk_window_get_geometry_hints(GdkWindow *window,
@@ -178,7 +139,7 @@ gdk_window_get_geometry_hints(GdkWindow *window,
 	}
 }
 
-gboolean terminal_window_resize(GtkWidget *widget, GtkRequisition *requisition, LXTerminal *terminal)
+static gboolean terminal_window_resize(GtkWidget *widget, GtkRequisition *requisition, LXTerminal *terminal)
 {
 	Term *term;
 	GdkGeometry hints;
@@ -220,7 +181,7 @@ gboolean terminal_window_resize(GtkWidget *widget, GtkRequisition *requisition,
 	return FALSE;
 }
 
-void terminal_window_set_fixed_size(LXTerminal *terminal)
+static void terminal_window_set_fixed_size(LXTerminal *terminal)
 {
 	Term *term;
 	gint i;
@@ -237,66 +198,14 @@ void terminal_window_set_fixed_size(LXTerminal *terminal)
 	}
 }
 
-gboolean terminal_switchtab1(LXTerminal *terminal)
+static gboolean term_switchtab(Term *term)
 {
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 0);
+	LXTerminal *terminal = term->parent;        
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), term->index);
 	return TRUE;
 }
 
-gboolean terminal_switchtab2(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 1);
-	return TRUE;
-}
-
-gboolean terminal_switchtab3(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 2);
-	return TRUE;
-}
-
-gboolean terminal_switchtab4(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 3);
-	return TRUE;
-}
-
-gboolean terminal_switchtab5(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 4);
-	return TRUE;
-}
-
-gboolean terminal_switchtab6(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 5);
-	return TRUE;
-}
-
-gboolean terminal_switchtab7(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 6);
-	return TRUE;
-}
-
-gboolean terminal_switchtab8(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 7);
-	return TRUE;
-}
-
-gboolean terminal_switchtab9(LXTerminal *terminal)
-{
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(terminal->notebook), 8);
-	return TRUE;
-}
-
-gboolean terminal_copy_accel(gpointer data, guint action, GtkWidget *item)
-{
-	return terminal_copy(NULL, data);
-}
-
-gboolean terminal_copy(GtkAction *action, gpointer data)
+static gboolean terminal_copy(GtkAction *action, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 	Term *term;
@@ -310,12 +219,12 @@ gboolean terminal_copy(GtkAction *action, gpointer data)
 	return TRUE;
 }
 
-gboolean terminal_paste_accel(gpointer data, guint action, GtkWidget *item)
+static gboolean terminal_copy_accel(gpointer data, guint action, GtkWidget *item)
 {
-	return terminal_paste(NULL, data);
+	return terminal_copy(NULL, data);
 }
 
-gboolean terminal_paste(GtkAction *action, gpointer data)
+static gboolean terminal_paste(GtkAction *action, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 	Term *term;
@@ -329,7 +238,12 @@ gboolean terminal_paste(GtkAction *action, gpointer data)
 	return TRUE;
 }
 
-void terminal_nexttab(GtkAction *action, gpointer data)
+static gboolean terminal_paste_accel(gpointer data, guint action, GtkWidget *item)
+{
+	return terminal_paste(NULL, data);
+}
+
+static void terminal_nexttab(GtkAction *action, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 
@@ -340,12 +254,12 @@ void terminal_nexttab(GtkAction *action, gpointer data)
 		gtk_notebook_next_page(GTK_NOTEBOOK(terminal->notebook));
 }
 
-void terminal_nexttab_accel(gpointer data, guint action, GtkWidget *item)
+static void terminal_nexttab_accel(gpointer data, guint action, GtkWidget *item)
 {
 	terminal_nexttab(NULL, data);
 }
 
-void terminal_prevtab(GtkAction *action, gpointer data)
+static void terminal_prevtab(GtkAction *action, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 
@@ -356,13 +270,13 @@ void terminal_prevtab(GtkAction *action, gpointer data)
 		gtk_notebook_prev_page(GTK_NOTEBOOK(terminal->notebook));
 }
 
-void terminal_prevtab_accel(gpointer data, guint action, GtkWidget *item)
+static void terminal_prevtab_accel(gpointer data, guint action, GtkWidget *item)
 {
 	terminal_prevtab(NULL, data);
 }
 
 /* direction: -1=left 1=right */
-void terminal_movetab(GtkAction *action, gpointer data, gint direction)
+static void terminal_movetab(GtkAction *action, gpointer data, gint direction)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 	GtkNotebook *notebook = GTK_NOTEBOOK(terminal->notebook);
@@ -375,32 +289,27 @@ void terminal_movetab(GtkAction *action, gpointer data, gint direction)
 		);
 }
 
-void terminal_movetableft(GtkAction *action, gpointer data)
+static void terminal_movetableft(GtkAction *action, gpointer data)
 {
 	terminal_movetab(action, data, -1);
 }
 
-void terminal_movetableft_accel(gpointer data, guint action, GtkWidget *item)
+static void terminal_movetableft_accel(gpointer data, guint action, GtkWidget *item)
 {
 	terminal_movetableft(NULL, data);
 }
 
-void terminal_movetabright(GtkAction *action, gpointer data)
+static void terminal_movetabright(GtkAction *action, gpointer data)
 {
 	terminal_movetab(action, data, 1);
 }
 
-void terminal_movetabright_accel(gpointer data, guint action, GtkWidget *item)
+static void terminal_movetabright_accel(gpointer data, guint action, GtkWidget *item)
 {
 	terminal_movetabright(NULL, data);
 }
 
-void terminal_closetab_accel(gpointer data, guint action, GtkWidget *item)
-{
-	terminal_closetab(NULL, data);
-}
-
-void terminal_closetab(GtkAction *action, gpointer data)
+static void terminal_closetab(GtkAction *action, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 	Term *term;
@@ -412,28 +321,75 @@ void terminal_closetab(GtkAction *action, gpointer data)
 	terminal_childexit(VTE_TERMINAL(term->vte), term);
 }
 
-void terminal_newwindow_accel(gpointer data, guint action, GtkWidget *item)
+static void terminal_closetab_accel(gpointer data, guint action, GtkWidget *item)
 {
-	terminal_newwindow(NULL, data);
+	terminal_closetab(NULL, data);
 }
 
-void terminal_newwindow(GtkAction *action, gpointer data)
+static void terminal_newwindow(GtkAction *action, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 
 	lxterminal_init(terminal->parent, 0, NULL, terminal->setting);
 }
 
-void terminal_newtab_accel(gpointer data, guint action, GtkWidget *item)
+static void terminal_newwindow_accel(gpointer data, guint action, GtkWidget *item)
 {
-	terminal_newtab(NULL, data);
+	terminal_newwindow(NULL, data);
 }
 
-void terminal_newtab(GtkWidget *widget, gpointer data)
+static void term_set_swicth_accel(Term *term)
+{
+	if (term->index + 1 < 10) {
+    	char switch_tab_accel[1 + 3 + 1 + 1 + 1]; /* "<ALT>n" */
+		guint key;
+		GdkModifierType mods;
+        GtkAccelGroup *accel_group = term->parent->menubar->accel_group;
+
+		sprintf(switch_tab_accel, "<ALT>%d", term->index + 1);
+		gtk_accelerator_parse(switch_tab_accel, &key, &mods);
+        term->closure = g_cclosure_new_swap(G_CALLBACK(term_switchtab), term, NULL);
+		gtk_accel_group_connect(accel_group, key, mods, GTK_ACCEL_LOCKED, 
+                                term->closure);
+	}
+}
+
+static void terminal_newtab(GtkWidget *widget, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
+	Term *term;
 
-	Term *term = terminal_new(terminal, _("LXTerminal"), g_get_current_dir(), NULL, NULL);
+#ifdef __linux
+	gchar cwd[PATH_MAX];
+
+	gint current = gtk_notebook_get_current_page(GTK_NOTEBOOK(terminal->notebook));
+	if (current != -1) {
+		gchar proc_cwd_link[PATH_MAX];
+		gchar *proc_cwd;
+		gint i;
+		for (i=0;i<terminal->terms->len;i++) {
+			term = g_ptr_array_index(terminal->terms, i);
+			if (term->index == current)
+				break;
+		}
+
+		g_message("%d", term->pid);
+		g_snprintf(proc_cwd_link, PATH_MAX, "/proc/%d/cwd", term->pid);
+		proc_cwd = g_file_read_link(proc_cwd_link, NULL);
+		if (proc_cwd) {
+			g_strlcpy(cwd, proc_cwd, PATH_MAX);
+			g_free(proc_cwd);
+		} else {
+			g_strlcpy(cwd, g_get_current_dir(), PATH_MAX);
+		}
+	} else {
+		g_strlcpy(cwd, g_get_current_dir(), PATH_MAX);
+	}
+
+	term = terminal_new(terminal, _("LXTerminal"), cwd, NULL, NULL);
+#else
+	term = terminal_new(terminal, _("LXTerminal"), g_get_current_dir(), NULL, NULL);
+#endif
 
 	/* add page to notebook */
 	gtk_notebook_append_page(GTK_NOTEBOOK(terminal->notebook), term->box, term->label->main);
@@ -452,6 +408,12 @@ void terminal_newtab(GtkWidget *widget, gpointer data)
 		/* show tab */
 		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(term->parent->notebook), TRUE);
 	}
+    term_set_swicth_accel(term);
+}
+
+static void terminal_newtab_accel(gpointer data, guint action, GtkWidget *item)
+{
+	terminal_newtab(NULL, data);
 }
 
 static void open_url( GtkAboutDialog* dlg, const gchar* url, gpointer data )
@@ -459,7 +421,7 @@ static void open_url( GtkAboutDialog* dlg, const gchar* url, gpointer data )
     /* FIXME: */
 }
 
-void terminal_about(GtkAction *action, gpointer data)
+static void terminal_about(GtkAction *action, gpointer data)
 {
     GtkWidget * about_dlg;
     const gchar *authors[] =
@@ -489,7 +451,46 @@ void terminal_about(GtkAction *action, gpointer data)
     gtk_widget_destroy( about_dlg );
 }
 
-void terminal_switch_tab(GtkNotebook *notebook, GtkNotebookPage *page, guint num, gpointer data)
+static GtkActionEntry menu_items[] =
+{
+	{ "File_NewWindow", GTK_STOCK_ADD, N_("New _Window"), NEW_WINDOW_ACCEL, "New Window", G_CALLBACK(terminal_newwindow)},
+	{ "File_NewTab", GTK_STOCK_ADD, N_("New _Tab"), NEW_TAB_ACCEL, "New Tab", G_CALLBACK(terminal_newtab)},
+	{ "File_Sep1", NULL, "Sep" },
+	{ "File_CloseTab", GTK_STOCK_CLOSE, N_("_Close Tab"), CLOSE_TAB_ACCEL, "Close Tab", G_CALLBACK(terminal_closetab)},
+	{ "File_Quit", GTK_STOCK_QUIT, N_("_Quit"), QUIT_ACCEL, "Quit", G_CALLBACK(gtk_main_quit)},
+	{ "Edit_Copy", GTK_STOCK_COPY, N_("Cop_y"), COPY_ACCEL, "Copy", G_CALLBACK(terminal_copy)},
+	{ "Edit_Paste", GTK_STOCK_PASTE, N_("_Paste"), PASTE_ACCEL, "Paste", G_CALLBACK(terminal_paste)},
+	{ "Edit_Sep1", NULL, "Sep" },
+	{ "Edit_Preferences", GTK_STOCK_EXECUTE, N_("Preference_s"), NULL, "Preferences", G_CALLBACK(lxterminal_preferences_dialog)},
+//	{ "View_CharacterEncoding", NULL, N_("_Character Encoding"), NULL, "Character Encoding", NULL},
+	{ "Tabs_PreviousTab", GTK_STOCK_GO_BACK, N_("Pre_vious Tab"), PREVIOUS_TAB_ACCEL, "Previous Tab", G_CALLBACK(terminal_prevtab)},
+	{ "Tabs_NextTab", GTK_STOCK_GO_FORWARD, N_("Ne_xt Tab"), NEXT_TAB_ACCEL, "Next Tab", G_CALLBACK(terminal_nexttab)},
+	{ "Tabs_MoveTabLeft", NULL, N_("Move Tab _Left"), MOVE_TAB_LEFT_ACCEL, "Move Tab Left", G_CALLBACK(terminal_movetableft)},
+	{ "Tabs_MoveTabRight", NULL, N_("Move Tab _Right"), MOVE_TAB_RIGHT_ACCEL, "Move Tab Right", G_CALLBACK(terminal_movetabright)},
+	{ "Help_About", GTK_STOCK_ABOUT, N_("_About"), NULL, "About", G_CALLBACK(terminal_about)}
+};
+#define MENUBAR_MENUITEM_COUNT G_N_ELEMENTS(menu_items)
+
+static GtkActionEntry vte_menu_items[] =
+{
+	{ "VTEMenu", NULL, "VTEMenu" },
+	{ "NewWindow", GTK_STOCK_ADD, N_("New _Window"), NULL, "New Window", G_CALLBACK(terminal_newwindow)},
+	{ "NewTab", GTK_STOCK_ADD, N_("New _Tab"), NULL, "New Tab", G_CALLBACK(terminal_newtab)},
+	{ "Sep1", NULL, "Sep" },
+	{ "Copy", GTK_STOCK_COPY, N_("Cop_y"), NULL, "Copy", G_CALLBACK(terminal_copy)},
+	{ "Paste", GTK_STOCK_PASTE, N_("_Paste"), NULL, "Paste", G_CALLBACK(terminal_paste)},
+	{ "Sep2", NULL, "Sep" },
+	{ "Preferences", GTK_STOCK_EXECUTE, N_("Preference_s"), NULL, "Preferences", G_CALLBACK(lxterminal_preferences_dialog)},
+	{ "Sep3", NULL, "Sep" },
+	{ "PreviousTab", GTK_STOCK_GO_BACK, N_("Pre_vious Tab"), NULL, "Previous Tab", G_CALLBACK(terminal_prevtab)},
+	{ "NextTab", GTK_STOCK_GO_FORWARD, N_("Ne_xt Tab"), NULL, "Next Tab", G_CALLBACK(terminal_nexttab)},
+	{ "Tabs_MoveTabLeft", NULL, N_("Move Tab _Left"), NULL, "Move Tab Left", G_CALLBACK(terminal_prevtab)},
+	{ "Tabs_MoveTabRight", NULL, N_("Move Tab _Right"), NULL, "Move Tab Right", G_CALLBACK(terminal_nexttab)},
+	{ "CloseTab", GTK_STOCK_CLOSE, N_("_Close Tab"), NULL, "Close Tab", G_CALLBACK(terminal_closetab)}
+};
+#define VTE_MENUITEM_COUNT G_N_ELEMENTS(vte_menu_items)
+
+static void terminal_switch_tab(GtkNotebook *notebook, GtkNotebookPage *page, guint num, gpointer data)
 {
 	const gchar *title;
 	Term *term;
@@ -507,16 +508,17 @@ void terminal_switch_tab(GtkNotebook *notebook, GtkNotebookPage *page, guint num
 		gtk_window_set_title(GTK_WINDOW(terminal->mainw), title);
 }
 
-void terminal_title_changed(VteTerminal *vte, Term *term)
+static void terminal_title_changed(VteTerminal *vte, Term *term)
 {
 	/* setting label */
 	lxterminal_tab_label_set_text(term->label, vte_terminal_get_window_title(VTE_TERMINAL(vte)));
+	lxterminal_tab_label_set_tooltip_text(term->label, vte_terminal_get_window_title(VTE_TERMINAL(vte)));
 
 	/* setting window title */
 	gtk_window_set_title(GTK_WINDOW(term->parent->mainw), vte_terminal_get_window_title(VTE_TERMINAL(vte)));
 }
 
-void terminal_windowexit(gpointer terminal_p, GObject *where_the_object_was)
+static void terminal_windowexit(gpointer terminal_p, GObject *where_the_object_was)
 {
 	LXTerminal * terminal = (LXTerminal *) terminal_p;
 	int i;
@@ -535,7 +537,7 @@ void terminal_windowexit(gpointer terminal_p, GObject *where_the_object_was)
 	}
 }
 
-void terminal_childexit(VteTerminal *vte, Term *term)
+static void terminal_childexit(VteTerminal *vte, Term *term)
 {
 	int i;
 	LXTerminal *terminal = term->parent;
@@ -558,7 +560,7 @@ void terminal_childexit(VteTerminal *vte, Term *term)
 
 		/* release */
 		gtk_notebook_remove_page(GTK_NOTEBOOK(terminal->notebook), term->index);
-		g_free(term);
+		terminal_free(term);
 
 		/* if only one page, hide tab */
 		if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(terminal->notebook)) == 1) {
@@ -588,7 +590,7 @@ void terminal_childexit(VteTerminal *vte, Term *term)
 	}
 }
 
-gboolean terminal_vte_button_press(VteTerminal *vte, GdkEventButton *event, gpointer data)
+static gboolean terminal_vte_button_press(VteTerminal *vte, GdkEventButton *event, gpointer data)
 {
 	LXTerminal *terminal = (LXTerminal *)data;
 	GtkWidget *menu_item;
@@ -617,7 +619,44 @@ gboolean terminal_vte_button_press(VteTerminal *vte, GdkEventButton *event, gpoi
 		}
 
 		gtk_menu_popup(GTK_MENU(gtk_ui_manager_get_widget(manager, "/VTEMenu")), NULL, NULL, NULL, NULL, event->button, event->time);
+	} else if (event->button == 1) { /* left click */
+		/* steal from tilda-0.09.6/src/tilda_terminal.c:743 */
+		gint tag;
+		gint xpad, ypad;
+		gchar* match;
+		gchar* cmd;
+		gboolean ret = FALSE;
+
+		vte_terminal_get_padding(vte, &xpad, &ypad);
+		match = vte_terminal_match_check(vte,
+				(event->x - xpad) / vte->char_width,
+				(event->y - ypad) / vte->char_height,
+				&tag);
+
+		/* Check if we can launch a web browser, and do so if possible */
+		if ((event->state & GDK_CONTROL_MASK) && match != NULL) {
+#if DEBUG
+			g_print("Got a Ctrl+Left Click -- Matched: `%s' (%d)\n", match, tag);
+#endif
+			cmd = g_strdup_printf("%s %s", "xdg-open", match);
+#if DEBUG
+			g_print("Launching command: `%s'\n", cmd);
+#endif
+			ret = g_spawn_command_line_async(cmd, NULL);
+
+			/* Check that the command launched */
+			if (!ret) {
+				g_printerr(_("Failed to launch the web browser. The command was `%s'\n"), cmd);
+			}
+
+			g_free(cmd);
+		}
+
+		/* Always free match if it is non NULL */
+		if (match)
+			g_free(match);
 	}
+
 #if 0
 	GtkItemFactory *item_factory;
 
@@ -632,8 +671,40 @@ gboolean terminal_vte_button_press(VteTerminal *vte, GdkEventButton *event, gpoi
 	return FALSE;
 }
 
-Term *terminal_new(LXTerminal *terminal, const gchar *label, const gchar *pwd, gchar **env, const gchar *exec)
+void terminal_term_setting_update(Term *term, LXTerminal *terminal){
+	vte_terminal_reset((VteTerminal *)term->vte, FALSE, FALSE);
+
+	vte_terminal_set_font_from_string((VteTerminal *)term->vte, terminal->setting->fontname);
+	vte_terminal_set_word_chars((VteTerminal *)term->vte, terminal->setting->selchars);
+	vte_terminal_set_scrollback_lines((VteTerminal *)term->vte, terminal->setting->scrollback);
+
+	vte_terminal_set_cursor_blink_mode((VteTerminal *)term->vte, terminal->setting->cursorblinks?VTE_CURSOR_BLINK_ON:VTE_CURSOR_BLINK_OFF);
+
+	/* background transparency */
+	if( terminal->rgba ){
+		/* dirty hack - vte_terminal_queue_background_update
+		 * doesn't run without changing background */
+		vte_terminal_set_color_background((VteTerminal *)term->vte, &terminal->foreground);
+		vte_terminal_set_background_transparent((VteTerminal *)term->vte, FALSE);
+		vte_terminal_set_opacity((VteTerminal *)term->vte, terminal->setting->bgalpha);
+	} else {
+		vte_terminal_set_background_transparent((VteTerminal *)term->vte, terminal->setting->bgalpha == 65535 ? FALSE : TRUE);
+		vte_terminal_set_background_saturation((VteTerminal *)term->vte, 1-((double)terminal->setting->bgalpha/65535));
+	}
+
+	vte_terminal_set_colors((VteTerminal *)term->vte, &terminal->foreground, &terminal->background, &linux_color[0], 16);
+
+	/* update scrollbar */
+	if( terminal->setting->hidescrollbar )
+		gtk_widget_hide( term->scrollbar );
+	else
+		gtk_widget_show( term->scrollbar );
+}
+
+static Term *terminal_new(LXTerminal *terminal, const gchar *label, const gchar *pwd, gchar **env, const gchar *exec)
 {
+	gint ret;
+	GRegex *dingus1, *dingus2;
 	Term *term;
 
 	/* create terminal */
@@ -646,9 +717,6 @@ Term *terminal_new(LXTerminal *terminal, const gchar *label, const gchar *pwd, g
 	gtk_box_pack_start(GTK_BOX(term->box), term->scrollbar, FALSE, TRUE, 0);
 
 	/* setting terminal */
-	vte_terminal_set_font_from_string((VteTerminal *)term->vte, terminal->setting->fontname);
-	vte_terminal_set_word_chars((VteTerminal *)term->vte, terminal->setting->selchars);
-	vte_terminal_set_scrollback_lines((VteTerminal *)term->vte, terminal->setting->scrollback);
 	vte_terminal_set_emulation((VteTerminal *)term->vte, "xterm");
 
 	/* Set encoding from locale. */
@@ -672,37 +740,36 @@ Term *terminal_new(LXTerminal *terminal, const gchar *label, const gchar *pwd, g
 		printf("Bad fgcolor string in config: %s\n", terminal->setting->fgcolor);
 	}
 
-	vte_terminal_set_colors(VTE_TERMINAL(term->vte), &terminal->foreground, &terminal->background, &linux_color[0], 16);
-
-	/* background transparent */
-	if( terminal->setting->bgtransparent ){
-		if( terminal->rgba ){
-			vte_terminal_set_background_transparent((VteTerminal *)term->vte, FALSE);
-			vte_terminal_set_opacity((VteTerminal *)term->vte, terminal->setting->bgalpha);
-		} else {
-			vte_terminal_set_background_transparent((VteTerminal *)term->vte, TRUE);
-			vte_terminal_set_background_saturation((VteTerminal *)term->vte, 1-((double)terminal->setting->bgalpha/65535));
-		}
-	} else {
-		vte_terminal_set_background_transparent((VteTerminal *)term->vte, FALSE);
-		vte_terminal_set_opacity((VteTerminal *)term->vte, 65535);
-	}
+	/* steal from tilda-0.09.6/src/tilda_terminal.c:145 */
+	/* Match URL's, etc */
+	dingus1 = g_regex_new(DINGUS1, G_REGEX_OPTIMIZE, 0, NULL);
+	dingus2 = g_regex_new(DINGUS2, G_REGEX_OPTIMIZE, 0, NULL);
+	ret = vte_terminal_match_add_gregex((VteTerminal *)term->vte, dingus1, 0);
+	vte_terminal_match_set_cursor_type((VteTerminal *)term->vte, ret, GDK_HAND2);
+	ret = vte_terminal_match_add_gregex((VteTerminal *)term->vte, dingus2, 0);
+	vte_terminal_match_set_cursor_type((VteTerminal *)term->vte, ret, GDK_HAND2);
+	g_regex_unref(dingus1);
+	g_regex_unref(dingus2);
 
 	/* create label for tab */
-	term->label = lxterminal_tab_label_new(label);
+	if (label)
+		term->label = lxterminal_tab_label_new(label);
+	else
+		term->label = lxterminal_tab_label_new(pwd);
+
 	lxterminal_tab_label_close_button_clicked(G_CALLBACK(terminal_childexit), term);
 
 	/* setting scrollbar */
 	gtk_range_set_adjustment(GTK_RANGE(term->scrollbar), VTE_TERMINAL(term->vte)->adjustment);
 
 	/* terminal fork */
-	if (!exec) {
-		vte_terminal_fork_command(VTE_TERMINAL(term->vte), NULL, NULL, env, pwd, FALSE, TRUE, TRUE);
-	} else {
+	if (exec) {
 		gchar **command;
 		g_shell_parse_argv(exec, NULL, &command, NULL);
-		vte_terminal_fork_command(VTE_TERMINAL(term->vte), (const char *)*(command), command, env, pwd, FALSE, TRUE, TRUE);
+		term->pid = vte_terminal_fork_command(VTE_TERMINAL(term->vte), (const char *)*(command), command, env, "/tmp", FALSE, TRUE, TRUE);
 		g_strfreev(command);
+	} else {
+		term->pid = vte_terminal_fork_command(VTE_TERMINAL(term->vte), NULL, NULL, env, pwd, FALSE, TRUE, TRUE);
 	}
 
 	/* signal handler */
@@ -712,10 +779,20 @@ Term *terminal_new(LXTerminal *terminal, const gchar *label, const gchar *pwd, g
 
 	gtk_widget_show_all(term->box);
 
+	/* apply other settings */
+	terminal_term_setting_update(term, terminal);
+
 	return term;
 }
 
-Menu *menubar_init(LXTerminal *terminal)
+void terminal_free(Term* term)
+{
+    GtkAccelGroup *accel_group = term->parent->menubar->accel_group;
+    gtk_accel_group_disconnect(accel_group, term->closure);
+    g_free(term);
+}
+
+static Menu *menubar_init(LXTerminal *terminal)
 {
 	Menu *menubar;
 	GtkWidget *menu_item;
@@ -789,7 +866,7 @@ Menu *menubar_init(LXTerminal *terminal)
 	return menubar;
 }
 
-void lxterminal_accelerator_init(LXTerminal *terminal)
+static void lxterminal_accelerator_init(LXTerminal *terminal)
 {
 	guint key;
 	GdkModifierType mods;
@@ -814,33 +891,6 @@ void lxterminal_accelerator_init(LXTerminal *terminal)
 	gtk_accelerator_parse(PASTE_ACCEL, &key, &mods);
 	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_paste_accel), terminal, NULL));
 
-	gtk_accelerator_parse(SWITCH_TAB1_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab1), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB2_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab2), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB3_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab3), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB4_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab4), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB5_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab5), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB6_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab6), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB7_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab7), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB8_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab8), terminal, NULL));
-
-	gtk_accelerator_parse(SWITCH_TAB9_ACCEL, &key, &mods);
-	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_switchtab9), terminal, NULL));
-
 	gtk_accelerator_parse(NEXT_TAB_ACCEL, &key, &mods);
 	gtk_accel_group_connect(terminal->menubar->accel_group, key, mods, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(terminal_nexttab_accel), terminal, NULL));
 
@@ -856,7 +906,7 @@ void lxterminal_accelerator_init(LXTerminal *terminal)
 	gtk_window_add_accel_group(GTK_WINDOW(terminal->mainw), terminal->menubar->accel_group);
 }
 
-void lxterminal_menuaccel_update(Setting *setting)
+void lxterminal_menuaccel_update(LXTerminal *terminal)
 {
 	/* update F10 status */
 	/* hack took from gnome-terminal */
@@ -872,7 +922,7 @@ void lxterminal_menuaccel_update(Setting *setting)
 	}
 
 	
-	if (setting->disablef10) {
+	if (terminal->setting->disablef10) {
 		gtk_settings_set_string_property (gtk_settings_get_default(),
 										"gtk-menu-bar-accel",
 										/* no one will ever press this ;-) */
@@ -886,41 +936,28 @@ void lxterminal_menuaccel_update(Setting *setting)
 	}
 }
 
-void terminal_setting_update(LXTerminal *terminal, Setting *setting)
+void terminal_setting_update(LXTerminal *terminal)
 {
-	Term *term;
 	gint i;
 
+	/* know if it is composited */
+	terminal->rgba = gtk_widget_is_composited( GTK_WIDGET(terminal->mainw) );
+
 	/* update all of terminals */
-	for (i=0;i<terminal->terms->len;i++) {
-		term = g_ptr_array_index(terminal->terms, i);
-
-		vte_terminal_set_font_from_string((VteTerminal *)term->vte, terminal->setting->fontname);
-		vte_terminal_set_word_chars((VteTerminal *)term->vte, terminal->setting->selchars);
-		vte_terminal_set_scrollback_lines((VteTerminal *)term->vte, terminal->setting->scrollback);
-
-		/* background transparent */
-		if( terminal->setting->bgtransparent ){
-			if( terminal->rgba ){
-				vte_terminal_set_background_transparent((VteTerminal *)term->vte, FALSE);
-				vte_terminal_set_opacity((VteTerminal *)term->vte, terminal->setting->bgalpha);
-			} else {
-				vte_terminal_set_background_transparent((VteTerminal *)term->vte, TRUE);
-				vte_terminal_set_background_saturation((VteTerminal *)term->vte, 1-((double)terminal->setting->bgalpha/65535));
-			}
-		} else {
-			vte_terminal_set_background_transparent((VteTerminal *)term->vte, FALSE);
-			vte_terminal_set_opacity((VteTerminal *)term->vte, 65535);
-		}
-
-		vte_terminal_set_colors((VteTerminal *)term->vte, &terminal->foreground, &terminal->background, &linux_color[0], 16);
-	}
+	for (i=0;i<terminal->terms->len;i++)
+		terminal_term_setting_update(g_ptr_array_index(terminal->terms, i), terminal);
 
 	/* update tab position */
 	lxterminal_tab_set_position(terminal->notebook, terminal->tabpos);
 
 	/* update menu accel */
-	lxterminal_menuaccel_update(setting);
+	lxterminal_menuaccel_update(terminal);
+
+	/* update menubar */
+	if( terminal->setting->hidemenubar )
+		gtk_widget_hide( terminal->menubar->menu );
+	else
+		gtk_widget_show( terminal->menubar->menu );
 }
 
 LXTerminal *lxterminal_init(LXTermWindow *lxtermwin, gint argc, gchar **argv, Setting *setting)
@@ -940,13 +977,13 @@ LXTerminal *lxterminal_init(LXTermWindow *lxtermwin, gint argc, gchar **argv, Se
 			if (strncmp(argv[i],"--command=", 10)==0) {
 				cmd = argv[i]+10;
 				continue;
-			} else if ((strcmp(argv[i],"--command")==0||strcmp(argv[i],"-e")==0)&&(i+1<argc)) {
+			} else if ((strcmp(argv[i],"--command")==0 || strcmp(argv[i],"-e") == 0) && (i+1<argc) ) {
 				cmd = argv[++i];
 				continue;
 			} else if (strncmp(argv[i],"--title=", 8)==0) {
 				title = argv[i]+8;
 				continue;
-			} else if ((strcmp(argv[i],"--title")==0||strcmp(argv[i],"-t")==0||strcmp(argv[i],"-T")==0)&&(i+1<argc)) {
+			} else if ((strcmp(argv[i],"--title")==0 || strcmp(argv[i],"-t")==0 || strcmp(argv[i],"-T")==0)&&(i+1<argc)) {
 				title = argv[++i];
 				continue;
 			} else if (strncmp(argv[i],"--working-directory=", 20)==0) {
@@ -955,6 +992,8 @@ LXTerminal *lxterminal_init(LXTermWindow *lxtermwin, gint argc, gchar **argv, Se
 			} else if (strncmp(argv[i],"--geometry=", 11)==0) {
 				sscanf(argv[i]+11, "%dx%d", &cols, &rows);
 				continue;
+			} else if ((strcmp(argv[i],"--loginshell")==0 || strcmp(argv[i],"-l")==0)&&cmd==NULL) {
+				cmd = "sh -l";
 			}
 		}
 	}
@@ -974,15 +1013,13 @@ LXTerminal *lxterminal_init(LXTermWindow *lxtermwin, gint argc, gchar **argv, Se
 	/* create window */
 	terminal->mainw = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
-	/* know if it is composited */
-	terminal->rgba = gtk_widget_is_composited( GTK_WIDGET(terminal->mainw) );
-
-	if (terminal->rgba){
-		GdkColormap *colormap = gdk_screen_get_rgba_colormap( gtk_widget_get_screen( GTK_WIDGET(terminal->mainw) ) );
-		if (colormap)
-			gtk_widget_set_colormap( GTK_WIDGET(terminal->mainw), colormap );
-	}
+	/* colormap assign */
+	GdkColormap *colormap = gdk_screen_get_rgba_colormap( gtk_widget_get_screen( GTK_WIDGET(terminal->mainw) ) );
+	if (colormap) gtk_widget_set_colormap( GTK_WIDGET(terminal->mainw), colormap );
 
+	/* when composited changed, reload the terminal settings */
+	g_signal_connect_swapped(GTK_WIDGET(terminal->mainw), "composited-changed", G_CALLBACK(terminal_setting_update), terminal);
+	
 	if (!title)
 		gtk_window_set_title(GTK_WINDOW(terminal->mainw), _("LXTerminal"));
 	else
@@ -1007,7 +1044,6 @@ LXTerminal *lxterminal_init(LXTermWindow *lxtermwin, gint argc, gchar **argv, Se
 
 	/* Tab Position */
 	terminal->tabpos = lxterminal_tab_get_position_id(terminal->setting->tabpos);
-	lxterminal_tab_set_position(terminal->notebook, terminal->tabpos);
 
 	g_signal_connect(terminal->notebook, "switch-page", G_CALLBACK(terminal_switch_tab), terminal);
 	gtk_box_pack_start(GTK_BOX(terminal->box), terminal->notebook, TRUE, TRUE, 0);
@@ -1026,15 +1062,18 @@ LXTerminal *lxterminal_init(LXTermWindow *lxtermwin, gint argc, gchar **argv, Se
 
 	/* initializing accelerator */
 	lxterminal_accelerator_init(terminal);
+        term_set_swicth_accel(term);
 
 	gtk_widget_show_all(terminal->mainw);
 
+	/* update terminal settings */
+	terminal_setting_update(terminal);
+
 	/* original hints of VTE */
 	gdk_window_get_geometry_hints(GTK_WIDGET(term->vte)->window,
 										&terminal->geometry,
 										&terminal->geom_mask);
 
-
 	/* resizing terminal with window size */
 	g_signal_connect(terminal->mainw, "size-request", G_CALLBACK(terminal_window_resize), terminal);
 
@@ -1072,6 +1111,8 @@ int main(gint argc, gchar** argv)
 				continue;
 			} else if (strncmp(argv[i],"--geometry=", 11)==0) {
 				continue;
+			} else if ((strcmp(argv[i],"--loginshell")==0 || strcmp(argv[i],"-l")==0)) {
+				continue;
 			}
 
 			printf("%s\n", helpmsg);
@@ -1106,9 +1147,6 @@ int main(gint argc, gchar** argv)
 	lxtermwin->windows = g_ptr_array_new();
 	lxtermwin->setting = setting;
 
-	/* initializing menu accelerator */
-	lxterminal_menuaccel_update(setting);
-
 	/* initializing LXTerminal */
 	lxterminal_init(lxtermwin, argc, argv, setting);
 
