@@ -28,21 +28,33 @@ typedef struct {
 	GtkWidget *box;
 	GtkWidget *font_label;
 	GtkWidget *font_button;
-	GtkWidget *selchars_label;
-	GtkWidget *selchars_entry;
 	GtkWidget *bgcolor_label;
 	GtkWidget *bgcolor_entry;
 	GtkWidget *fgcolor_label;
 	GtkWidget *fgcolor_entry;
-	GtkWidget *bgtransparent_label;
-	GtkWidget *bgtransparent_checkbox;
-	GtkWidget *scrollback_label;
-	GtkWidget *scrollback_entry;
+	GtkWidget *cursorblinks_label;
+	GtkWidget *cursorblinks_checkbox;
+} PreferStyle;
+
+typedef struct {
+	GtkWidget *box;
 	GtkWidget *tabpos_label;
 	GtkWidget *tabpos_combobox;
+	GtkWidget *scrollback_label;
+	GtkWidget *scrollback_entry;
+	GtkWidget *hidescrollbar_label;
+	GtkWidget *hidescrollbar_checkbox;
+	GtkWidget *hidemenubar_label;
+	GtkWidget *hidemenubar_checkbox;
+} PreferDisplay;
+
+typedef struct {
+	GtkWidget *box;
+	GtkWidget *selchars_label;
+	GtkWidget *selchars_entry;
 	GtkWidget *disablef10_label;
 	GtkWidget *disablef10_checkbox;
-} PreferGeneral;
+} PreferMisc;
 
 void lxterminal_preferences_dialog(GtkAction *action, gpointer data);
 
