@@ -32,8 +32,11 @@ void setting_save_to_file(const char *path, const char *data)
 
 	/* open config file */
 	fp = fopen(path, "w");
-	fputs(data, fp);
-	fclose(fp);
+	if (fp != NULL)
+	{
+	    fputs(data, fp);
+	    fclose(fp);
+	}
 }
 
 void setting_save(Setting *setting)
@@ -50,10 +53,12 @@ void setting_save(Setting *setting)
 	g_key_file_set_string(setting->keyfile, "general", "bgcolor", setting->bgcolor);
 	g_key_file_set_integer(setting->keyfile, "general", "bgalpha", setting->bgalpha);
 	g_key_file_set_string(setting->keyfile, "general", "fgcolor", setting->fgcolor);
-	g_key_file_set_boolean(setting->keyfile, "general", "bgtransparent", setting->bgtransparent);
 	g_key_file_set_string(setting->keyfile, "general", "tabpos", setting->tabpos);
 	g_key_file_set_integer(setting->keyfile, "general", "scrollback", (gint)setting->scrollback);
 	g_key_file_set_boolean(setting->keyfile, "general", "disablef10", setting->disablef10);
+	g_key_file_set_boolean(setting->keyfile, "general", "hidemenubar", setting->hidemenubar);
+	g_key_file_set_boolean(setting->keyfile, "general", "hidescrollbar", setting->hidescrollbar);
+	g_key_file_set_boolean(setting->keyfile, "general", "cursorblinks", setting->cursorblinks);
 
 	/* generate config data */
 	file_data = g_key_file_to_data(setting->keyfile, NULL, NULL);
@@ -88,10 +93,12 @@ Setting *load_setting_from_file(const char *filename)
 		setting->bgcolor = g_key_file_get_string(setting->keyfile, "general", "bgcolor", NULL);
 		setting->bgalpha = g_key_file_get_integer(setting->keyfile, "general", "bgalpha", NULL);
 		setting->fgcolor = g_key_file_get_string(setting->keyfile, "general", "fgcolor", NULL);
-		setting->bgtransparent = g_key_file_get_boolean(setting->keyfile, "general", "bgtransparent", NULL);
 		setting->tabpos = g_key_file_get_string(setting->keyfile, "general", "tabpos", NULL);
 		setting->scrollback = (glong)g_key_file_get_integer(setting->keyfile, "general", "scrollback", NULL);
 		setting->disablef10 = g_key_file_get_boolean(setting->keyfile, "general", "disablef10", NULL);
+		setting->hidemenubar = g_key_file_get_boolean(setting->keyfile, "general", "hidemenubar", NULL);
+		setting->hidescrollbar = g_key_file_get_boolean(setting->keyfile, "general", "hidescrollbar", NULL);
+		setting->cursorblinks = g_key_file_get_boolean(setting->keyfile, "general", "cursorblinks", NULL);
 	}
 
 setting_default:
@@ -106,14 +113,11 @@ setting_default:
 		setting->bgcolor = g_strdup("#000000");
 	
 	if (!setting->bgalpha)
-		setting->bgalpha = (guint16) 0xDDDD;
+		setting->bgalpha = (guint16) 0xFFFF;
 
 	if (!setting->fgcolor)
 		setting->fgcolor = g_strdup("#aaaaaa");
 
-	if (!setting->bgtransparent)
-		setting->bgtransparent = FALSE;
-
 	if (!setting->tabpos)
 		setting->tabpos = g_strdup("top");
 
@@ -123,5 +127,14 @@ setting_default:
 	if (!setting->disablef10)
 		setting->disablef10 = FALSE;
 
+	if (!setting->hidemenubar)
+		setting->hidemenubar = FALSE;
+
+	if (!setting->hidescrollbar)
+		setting->hidescrollbar = FALSE;
+
+	if (!setting->cursorblinks)
+		setting->cursorblinks = FALSE;
+
 	return setting;
 }
