@@ -10,8 +10,10 @@ typedef struct _setting {
 	guint16 bgalpha;
 	glong scrollback;
 	gboolean disablef10;
-	gboolean bgtransparent;
 	GKeyFile *keyfile;
+	gboolean hidemenubar;
+	gboolean hidescrollbar;
+	gboolean cursorblinks;
 } Setting;
 
 Setting *load_setting_from_file(const char *filename);
