@@ -174,7 +174,7 @@ lxterminal_socket_init(LXTermWindow *lxtermwin, int argc, char **argv)
 				setworkdir = TRUE;
 			}
 
-			g_io_channel_write_chars(gio, *(argv+i), -1, NULL, NULL);
+			g_io_channel_write_chars(gio, g_shell_quote(*(argv+i)), -1, NULL, NULL);
 			if (i+1!=argc) {
 				g_io_channel_write_chars(gio, " ", -1, NULL, NULL);
 			} else {
